

local moduleName = "Dispel"



----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBars"..moduleName)

L:RegisterTranslations("enUS", function() return {
	[moduleName] = "Dispel",

	["Debuffs"] = true,
	["Enable/Disable debuffs."] = true,

	["Add debuff"] = true,
	["Add a debuff to show. Must match the exact name of the debuff, including capital case."] = true,

	["Sound Warning"] = true,
	["Play a sound when there is something to dispel."] = true,

	["Enable/Disable showing of "] = true,
	["[%s] added to the debuff list."] = true
} end)



----------------------------------
--      Local Declaration      --
----------------------------------

local BS = AceLibrary("Babble-Spell-2.2")

local candispel = {
	["WARRIOR"] = { },
	["ROGUE"] = { },
	["HUNTER"] = { },
	["MAGE"] = {
		["Curse"] = true
	},
	["WARLOCK"] = { },
	["DRUID"] = { 
		["Curse"] = true,
		["Poison"] = true
	},
	["PALADIN"] = {
		["Magic"] = true,
		["Poison"] = true,
		["Disease"] = true
	},
	["PRIEST"] = {
		["Magic"] = true,
		["Disease"] = true
	},
	["SHAMAN"] = {
		["Poison"] = true,
		["Disease"] = true
	}
}

do
	local class = select(2, UnitClass("player"))
	candispel = candispel[class] -- let's hope the rest of the candispel table is garbage collected after this
	candispel["none"] = true
end


local ignore = {
	-- debuffs here should never be dispelled
	[BS["Dreamless Sleep"]] = true,
	[BS["Greater Dreamless Sleep"]] = true,
	[BS["Major Dreamless Sleep"]] = true,

	[BS["Delusions of Jin'do"]] = true,

	[BS["Unstable Affliction"]] = true,

	-- debuffs here can be ignored
	["WARRIOR"] = {
		[BS["Ancient Hysteria"]] = true, -- Spirit reduced by 50%, Intellect reduced by 50%.
		[BS["Ignite Mana"]] = true, -- 400 mana burned every 3 sec.
		[BS["Tainted Mind"]] = true, -- Intellect reduced by 20%.
		[BS["Curse of Tongues"]] = true -- Casting speed reduced by 75%.
	},
	["ROGUE"] = {
		[BS["Ancient Hysteria"]] = true, -- Spirit reduced by 50%, Intellect reduced by 50%.
		[BS["Ignite Mana"]] = true, -- 400 mana burned every 3 sec.
		[BS["Tainted Mind"]] = true, -- Intellect reduced by 20%.
		[BS["Curse of Tongues"]] = true, -- Casting speed reduced by 75%.
		[BS["Silence"]] = true, -- Cannot cast spells.
		[BS["Sonic Burst"]] = true -- Cannot cast spells.
	},
	["HUNTER"] = {
	},
	["MAGE"] = {
		[BS["Dust Cloud"]] = true -- Chance to hit reduced by 30%.
	},
	["WARLOCK"] = {
		[BS["Dust Cloud"]] = true -- Chance to hit reduced by 30%.
	},
	["DRUID"] = {
	},
	["PALADIN"] = {
	},
	["PRIEST"] = {
		[BS["Dust Cloud"]] = true -- Chance to hit reduced by 30%.
	},
	["SHAMAN"] = {
	}
}

local sound = "Sound\\Doodad\\BellTollNightElf.wav" -- Simple ding


-- localize these functions to speed up the main loop a bit
local UnitClass = UnitClass
local UnitDebuff = UnitDebuff
local PlaySound = PlaySound
local DebuffTypeColor = DebuffTypeColor


local debuffs = {}



----------------------------------
--      Module Declaration      --
----------------------------------

local plugin = BunchOfBars:NewModule(moduleName, "AceEvent-2.0")

plugin.revision = tonumber(("$Revision: 39274 $"):match("%d+"))

plugin.options = {
	name = L[moduleName],
	args = {
		debuffs = {
			type = "group",
			name = L["Debuffs"],
			desc = L["Enable/Disable debuffs."],
			args = { }
		},
		add = {
			type  = "text",
			name  = L["Add debuff"],
			desc  = L["Add a debuff to show. Must match the exact name of the debuff, including capital case."],
			usage = "",
			get   = function() return "" end,
			set   = "NewDebuff"
		},
		sound = {
			type = "toggle",
			name = L["Sound Warning"],
			desc = L["Play a sound when there is something to dispel."],
            get  = "GetSetSound",
            set  = "GetSetSound"
		}
	}
}

plugin.defaultDB = {
	sound   = true,
	debuffs = {
		[BS["Water Tomb"]]    = true, -- Hydros
		[BS["Watery Grave"]]  = true, -- Morogrim
		[BS["Frost Blast"]]   = true, -- Kel'thuzad
		[BS["Garrote"]]       = true, -- Moroes
		[BS["Mortal Strike"]] = true
	}
}



----------------------------------
--      Module Functions        --
----------------------------------

function plugin:OnEnable()
	-- TODO: Need to reset the menu on profile reset
	for n,_ in pairs(self.db.profile.debuffs) do
		self:NewDebuff(n, true)
	end

	self:ScheduleRepeatingEvent(self.UpdateAll, 0.5, self) -- TODO: test if "UNIT_AURA" works as nicely
end


function plugin:OnCreate(frame)
	local highlight = frame:CreateTexture(nil, "BACKGROUND")
	highlight:SetTexture("Interface/Tooltips/UI-Tooltip-Background")
	highlight:ClearAllPoints()
	highlight:SetAllPoints(frame)
	highlight:Hide()

	return highlight
end


function plugin:OnUpdate(frame, highlight)
	local class = select(2, UnitClass(frame.unit))

	local hadone = highlight:IsShown()
	highlight:Hide()

	for k in pairs(debuffs) do debuffs[k] = nil end -- TODO: debuffs is a local variable which seems to decrease memory usage by a lot, verify this.

	for i = 1,32 do
		local name, _, _, _, type = UnitDebuff(frame.unit, i)

		if not name then break end

		if self.db.profile.debuffs[name] then
			debuffs["none"] = name
		elseif candispel[type] then
			if ignore[name] then
				debuffs[type] = "nodispel"
			elseif not ignore[class][name] and debuffs[type] ~= "nodispel" then
				debuffs[type] = name
			end
		end
	end

	for type in pairs(candispel) do
		if debuffs[type] and debuffs[type] ~= "nodispel" then
			local inrange = CheckInteractDistance(frame.unit, 1) -- 30 yards

			highlight:SetVertexColor(DebuffTypeColor[type].r, DebuffTypeColor[type].g, DebuffTypeColor[type].b, inrange and 1.0 or 0.75)
			highlight:Show()

			if inrange and not hadone and self.db.profile.sound then
				PlaySoundFile(sound)
			end
			return
		end
	end
end


function plugin:NewDebuff(n, menuonly)
	n = strtrim(n)

	if n ~= "" then
		self.options.args.debuffs.args[n] = {
			type = "toggle",
			name = n,
			desc = L["Enable/Disable showing of "]..n,
			get  = function() return plugin.db.profile.debuffs[n] end,
			set  = function(v) plugin.db.profile.debuffs[n] = v end
		}

		if not menuonly then
			self.db.profile.debuffs[n] = true

			self.core:Print(string.format(L["[%s] added to the debuff list."], n))
		end
	end
end


function plugin:GetSetSound(v)
	if type(v) == "nil" then return self.db.profile.sound end

	self.db.profile.sound = v

	if v then PlaySoundFile(sound) end
end
