

local moduleName = "Hots"



----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBars"..moduleName)

L:RegisterTranslations("enUS", function() return {
	[moduleName] = "Hots",

	["Always"] = true,
	["Enable/Disable buffs which are always shown."] = true,

	["Out Of Combat"] = true,
	["Enable/Disable Out of combat buffs."] = true,

	["Text Color"] = true,
	["Change the text color."] = true,

	["Enable/Disable showing of "] = true
} end)



----------------------------------
--      Local Declaration      --
----------------------------------

local BS = AceLibrary("Babble-Spell-2.2")

local buffs = {
	{"DRUID", BS["Rejuvenation"]},
	{"DRUID", BS["Regrowth"]},
	{"DRUID", BS["Lifebloom"]},
	{"DRUID", BS["Mark of the Wild"]},
	{"DRUID", BS["Gift of the Wild"]},
	{"DRUID", BS["Thorns"]},
	{"DRUID", BS["Innervate"]},
	{"DRUID", BS["Tree of Life"]},

	{"PRIEST", BS["Power Word: Fortitude"]},
	{"PRIEST", BS["Prayer of Fortitude"]},
	{"PRIEST", BS["Power Word: Shield"]},
	{"PRIEST", BS["Renew"]},
	{"PRIEST", BS["Shadow Protection"]},
	{"PRIEST", BS["Divine Spirit"]},
	{"PRIEST", BS["Fear Ward"]},
	{"PRIEST", BS["Prayer of Mending"]},

	{"PALADIN", BS["Blessing of Freedom"]},
	{"PALADIN", BS["Blessing of Kings"]},
	{"PALADIN", BS["Blessing of Light"]},
	{"PALADIN", BS["Blessing of Might"]},
	{"PALADIN", BS["Blessing of Protection"]},
	{"PALADIN", BS["Blessing of Sacrifice"]},
	{"PALADIN", BS["Blessing of Salvation"]},
	{"PALADIN", BS["Blessing of Sanctuary"]},
	{"PALADIN", BS["Blessing of Wisdom"]},
	{"PALADIN", BS["Greater Blessing of Kings"]},
	{"PALADIN", BS["Greater Blessing of Light"]},
	{"PALADIN", BS["Greater Blessing of Might"]},
	{"PALADIN", BS["Greater Blessing of Salvation"]},
	{"PALADIN", BS["Greater Blessing of Sanctuary"]},
	{"PALADIN", BS["Greater Blessing of Wisdom"]},

	{"MAGE", BS["Arcane Intellect"]},

	{"WARLOCK", BS["Soulstone Resurrection"]},
	{"WARLOCK", BS["Blood Pact"]},

	{"WARRIOR", BS["Shield Wall"]}
}

local incombat = false

local hotcount = 5

-- localize these functions to speed up our main loop
local UnitBuff = UnitBuff
local UnitAffectingCombat = UnitAffectingCombat



----------------------------------
--      Module Declaration      --
----------------------------------

local plugin = BunchOfBars:NewModule(moduleName, "AceEvent-2.0")

plugin.revision = tonumber(("$Revision: 39274 $"):match("%d+"))

plugin.options = {
	name = L[moduleName],
	args = {
		always = {
			type = "group",
			name = L["Always"],
			desc = L["Enable/Disable buffs which are always shown."],
			args = { }
		},
		ooc = {
			type = "group",
			name = L["Out Of Combat"],
			desc = L["Enable/Disable Out of combat buffs."],
			args = { }
		},
		color = {
			type = "color",
			name = L["Text Color"],
			desc = L["Change the text color."],
			get = "GetSetColor",
			set = "GetSetColor"
		}
	}
}

plugin.defaultDB = {
	position = 3,

	always = {}, -- these are automatically filled
	ooc    = {},
	color  = {
		r = 1,
		g = 1,
		b = 0
	}
}


do -- let's not pollute our name space
	local _,class = UnitClass("player")

	for i,a in ipairs(buffs) do
		local cname = "|cff"..string.format("%02x%02x%02x", RAID_CLASS_COLORS[a[1]].r * 255, RAID_CLASS_COLORS[a[1]].g * 255, RAID_CLASS_COLORS[a[1]].b * 255)..a[2].."|r"

		plugin.options.args.always.args[a[2]] = {
			type  = "toggle",
			name  = cname,
			desc  = L["Enable/Disable showing of "]..a[2],
			get   = function() return plugin.db.profile.always[a[2]] end,
			set   = function(v) plugin.db.profile.always[a[2]] = v end,
			order = i
		}

		plugin.options.args.ooc.args[a[2]] = {
			type  = "toggle",
			name  = cname,
			desc  = L["Enable/Disable showing of "]..a[2],
			get   = function() return plugin.db.profile.ooc[a[2]] end,
			set   = function(v) plugin.db.profile.ooc[a[2]] = v end,
			order = i
		}

		if a[1] == class then
			if class == "PALADIN" then
				plugin.defaultDB.always[a[2]] = true
			else
				plugin.defaultDB.ooc[a[2]] = true
			end
		end
	end
end



----------------------------------
--      Module Functions        --
----------------------------------

function plugin:OnEnable()
	self:ScheduleRepeatingEvent(self.Update, 0.5, self)
end


function plugin:OnCreate(frame)
	local buffbar = CreateFrame("Frame", nil, frame)
	buffbar:SetWidth(hotcount * (18 + 2))
	buffbar:SetHeight(18)

	local t
	for i = 1,hotcount do
		t = buffbar:CreateTexture(nil, "ARTWORK")
		t:SetHeight(16)
		t:SetWidth(16)
		t:SetVertexColor(1, 1, 1, 0.6)
		t:SetTexCoord(0.1, 0.9, 0.1, 0.9)
		t:ClearAllPoints()
		t:SetPoint("LEFT", buffbar, "LEFT", (i - 1) * 18, 0)
		t:Hide()
		buffbar[i] = t

		local font = buffbar:CreateFontString(nil, "OVERLAY")
		font:SetFontObject(GameFontNormalSmall)
		font:SetTextColor(self.db.profile.color.r ,self.db.profile.color.g, self.db.profile.color.b, 1)
		font:ClearAllPoints()
		font:SetAllPoints(t)
		font:Hide()
		t.font = font

		--local cooldown = CreateFrame("Cooldown", nil, buffbar)
		--cooldown:SetReverse(true)
		--cooldown:ClearAllPoints()
		--cooldown:SetAllPoints(t)
		--cooldown:Hide()
		--t.cooldown = cooldown
	end

	return buffbar
end


function plugin:OnUpdate(frame, buffbar)
	local n = 1

	for i = 1,32 do
		local name, _, texture, applications, duration, timeleft = UnitBuff(frame.unit, i)

		if not name then break end

		if self.db.profile.always[name] or (not incombat and self.db.profile.ooc[name]) then
			buffbar[n]:SetTexture(texture)
			buffbar[n]:Show()

			if timeleft and timeleft > 0 then
				--buffbar[n].cooldown:SetCooldown(GetTime() + timeleft - duration, duration)
				--buffbar[n].cooldown:Show()

				if timeleft > 60 then timeleft = timeleft / 60 end
				timeleft = math.floor(timeleft)

				buffbar[n].font:SetText(timeleft)
				buffbar[n].font:Show()				
			else
				buffbar[n].font:Hide()
				--buffbar[n].cooldown:Hide()
			end

			n = n + 1
			if n > hotcount then break end
		end
	end

	for i = n,hotcount do
		buffbar[i]:Hide()
		buffbar[i].font:Hide()
		--buffbar[i].cooldown:Hide()
	end
end


function plugin:Update()
	incombat = UnitAffectingCombat("player") or UnitAffectingCombat("target")

	self:UpdateAll()
end


function plugin:GetSetColor(r, g, b)
	if type(r) == "nil" then return self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b end

	self.db.profile.color.r = r
	self.db.profile.color.g = g
	self.db.profile.color.b = b

	for _,frame in pairs(self.core.frames) do
		for i = 1,hotcount do
			frame.parts[self.name][i].font:SetTextColor(self.db.profile.color.r ,self.db.profile.color.g, self.db.profile.color.b, 1)
		end
	end
end

